@@ -1,8 +1,8 @@
-/* 
+/*
  * Copyright © 2008-2011 Kristian Høgsberg
  * Copyright © 2010-2011 Intel Corporation
  * Copyright © 2012-2013 Collabora, Ltd.
- * 
+ *
  * Permission to use, copy, modify, distribute, and sell this
  * software and its documentation for any purpose is hereby granted
  * without fee, provided that the above copyright notice appear in
@@ -14,7 +14,7 @@
  * representations about the suitability of this software for any
  * purpose.  It is provided "as is" without express or implied
  * warranty.
- * 
+ *
  * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
  * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
  * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
@@ -158,6 +158,9 @@ wl_display_add_listener(struct wl_display *wl_display,
 #define WL_DISPLAY_SYNC	0
 #define WL_DISPLAY_GET_REGISTRY	1
 
+#define WL_DISPLAY_SYNC_SINCE_VERSION	1
+#define WL_DISPLAY_GET_REGISTRY_SINCE_VERSION	1
+
 static inline void
 wl_display_set_user_data(struct wl_display *wl_display, void *user_data)
 {
@@ -170,6 +173,12 @@ wl_display_get_user_data(struct wl_display *wl_display)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_display);
 }
 
+static inline uint32_t
+wl_display_get_version(struct wl_display *wl_display)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_display);
+}
+
 static inline struct wl_callback *
 wl_display_sync(struct wl_display *wl_display)
 {
@@ -262,6 +271,8 @@ wl_registry_add_listener(struct wl_registry *wl_registry,
 
 #define WL_REGISTRY_BIND	0
 
+#define WL_REGISTRY_BIND_SINCE_VERSION	1
+
 static inline void
 wl_registry_set_user_data(struct wl_registry *wl_registry, void *user_data)
 {
@@ -274,6 +285,12 @@ wl_registry_get_user_data(struct wl_registry *wl_registry)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_registry);
 }
 
+static inline uint32_t
+wl_registry_get_version(struct wl_registry *wl_registry)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_registry);
+}
+
 static inline void
 wl_registry_destroy(struct wl_registry *wl_registry)
 {
@@ -285,8 +302,8 @@ wl_registry_bind(struct wl_registry *wl_registry, uint32_t name, const struct wl
 {
 	struct wl_proxy *id;
 
-	id = wl_proxy_marshal_constructor((struct wl_proxy *) wl_registry,
-			 WL_REGISTRY_BIND, interface, name, interface->name, version, NULL);
+	id = wl_proxy_marshal_constructor_versioned((struct wl_proxy *) wl_registry,
+			 WL_REGISTRY_BIND, interface, version, name, interface->name, version, NULL);
 
 	return (void *) id;
 }
@@ -318,6 +335,7 @@ wl_callback_add_listener(struct wl_callback *wl_callback,
 				     (void (**)(void)) listener, data);
 }
 
+
 static inline void
 wl_callback_set_user_data(struct wl_callback *wl_callback, void *user_data)
 {
@@ -330,6 +348,12 @@ wl_callback_get_user_data(struct wl_callback *wl_callback)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_callback);
 }
 
+static inline uint32_t
+wl_callback_get_version(struct wl_callback *wl_callback)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_callback);
+}
+
 static inline void
 wl_callback_destroy(struct wl_callback *wl_callback)
 {
@@ -339,6 +363,9 @@ wl_callback_destroy(struct wl_callback *wl_callback)
 #define WL_COMPOSITOR_CREATE_SURFACE	0
 #define WL_COMPOSITOR_CREATE_REGION	1
 
+#define WL_COMPOSITOR_CREATE_SURFACE_SINCE_VERSION	1
+#define WL_COMPOSITOR_CREATE_REGION_SINCE_VERSION	1
+
 static inline void
 wl_compositor_set_user_data(struct wl_compositor *wl_compositor, void *user_data)
 {
@@ -351,6 +378,12 @@ wl_compositor_get_user_data(struct wl_compositor *wl_compositor)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_compositor);
 }
 
+static inline uint32_t
+wl_compositor_get_version(struct wl_compositor *wl_compositor)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_compositor);
+}
+
 static inline void
 wl_compositor_destroy(struct wl_compositor *wl_compositor)
 {
@@ -383,6 +416,10 @@ wl_compositor_create_region(struct wl_compositor *wl_compositor)
 #define WL_SHM_POOL_DESTROY	1
 #define WL_SHM_POOL_RESIZE	2
 
+#define WL_SHM_POOL_CREATE_BUFFER_SINCE_VERSION	1
+#define WL_SHM_POOL_DESTROY_SINCE_VERSION	1
+#define WL_SHM_POOL_RESIZE_SINCE_VERSION	1
+
 static inline void
 wl_shm_pool_set_user_data(struct wl_shm_pool *wl_shm_pool, void *user_data)
 {
@@ -395,6 +432,12 @@ wl_shm_pool_get_user_data(struct wl_shm_pool *wl_shm_pool)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_shm_pool);
 }
 
+static inline uint32_t
+wl_shm_pool_get_version(struct wl_shm_pool *wl_shm_pool)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_shm_pool);
+}
+
 static inline struct wl_buffer *
 wl_shm_pool_create_buffer(struct wl_shm_pool *wl_shm_pool, int32_t offset, int32_t width, int32_t height, int32_t stride, uint32_t format)
 {
@@ -605,6 +648,8 @@ wl_shm_add_listener(struct wl_shm *wl_shm,
 
 #define WL_SHM_CREATE_POOL	0
 
+#define WL_SHM_CREATE_POOL_SINCE_VERSION	1
+
 static inline void
 wl_shm_set_user_data(struct wl_shm *wl_shm, void *user_data)
 {
@@ -617,6 +662,12 @@ wl_shm_get_user_data(struct wl_shm *wl_shm)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_shm);
 }
 
+static inline uint32_t
+wl_shm_get_version(struct wl_shm *wl_shm)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_shm);
+}
+
 static inline void
 wl_shm_destroy(struct wl_shm *wl_shm)
 {
@@ -675,6 +726,8 @@ wl_buffer_add_listener(struct wl_buffer *wl_buffer,
 
 #define WL_BUFFER_DESTROY	0
 
+#define WL_BUFFER_DESTROY_SINCE_VERSION	1
+
 static inline void
 wl_buffer_set_user_data(struct wl_buffer *wl_buffer, void *user_data)
 {
@@ -687,6 +740,12 @@ wl_buffer_get_user_data(struct wl_buffer *wl_buffer)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_buffer);
 }
 
+static inline uint32_t
+wl_buffer_get_version(struct wl_buffer *wl_buffer)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_buffer);
+}
+
 static inline void
 wl_buffer_destroy(struct wl_buffer *wl_buffer)
 {
@@ -731,6 +790,10 @@ wl_data_offer_add_listener(struct wl_data_offer *wl_data_offer,
 #define WL_DATA_OFFER_RECEIVE	1
 #define WL_DATA_OFFER_DESTROY	2
 
+#define WL_DATA_OFFER_ACCEPT_SINCE_VERSION	1
+#define WL_DATA_OFFER_RECEIVE_SINCE_VERSION	1
+#define WL_DATA_OFFER_DESTROY_SINCE_VERSION	1
+
 static inline void
 wl_data_offer_set_user_data(struct wl_data_offer *wl_data_offer, void *user_data)
 {
@@ -743,6 +806,12 @@ wl_data_offer_get_user_data(struct wl_data_offer *wl_data_offer)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_data_offer);
 }
 
+static inline uint32_t
+wl_data_offer_get_version(struct wl_data_offer *wl_data_offer)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_data_offer);
+}
+
 static inline void
 wl_data_offer_accept(struct wl_data_offer *wl_data_offer, uint32_t serial, const char *mime_type)
 {
@@ -824,6 +893,9 @@ wl_data_source_add_listener(struct wl_data_source *wl_data_source,
 #define WL_DATA_SOURCE_OFFER	0
 #define WL_DATA_SOURCE_DESTROY	1
 
+#define WL_DATA_SOURCE_OFFER_SINCE_VERSION	1
+#define WL_DATA_SOURCE_DESTROY_SINCE_VERSION	1
+
 static inline void
 wl_data_source_set_user_data(struct wl_data_source *wl_data_source, void *user_data)
 {
@@ -836,6 +908,12 @@ wl_data_source_get_user_data(struct wl_data_source *wl_data_source)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_data_source);
 }
 
+static inline uint32_t
+wl_data_source_get_version(struct wl_data_source *wl_data_source)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_data_source);
+}
+
 static inline void
 wl_data_source_offer(struct wl_data_source *wl_data_source, const char *mime_type)
 {
@@ -966,6 +1044,9 @@ wl_data_device_add_listener(struct wl_data_device *wl_data_device,
 #define WL_DATA_DEVICE_START_DRAG	0
 #define WL_DATA_DEVICE_SET_SELECTION	1
 
+#define WL_DATA_DEVICE_START_DRAG_SINCE_VERSION	1
+#define WL_DATA_DEVICE_SET_SELECTION_SINCE_VERSION	1
+
 static inline void
 wl_data_device_set_user_data(struct wl_data_device *wl_data_device, void *user_data)
 {
@@ -978,6 +1059,12 @@ wl_data_device_get_user_data(struct wl_data_device *wl_data_device)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_data_device);
 }
 
+static inline uint32_t
+wl_data_device_get_version(struct wl_data_device *wl_data_device)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_data_device);
+}
+
 static inline void
 wl_data_device_destroy(struct wl_data_device *wl_data_device)
 {
@@ -1001,6 +1088,9 @@ wl_data_device_set_selection(struct wl_data_device *wl_data_device, struct wl_da
 #define WL_DATA_DEVICE_MANAGER_CREATE_DATA_SOURCE	0
 #define WL_DATA_DEVICE_MANAGER_GET_DATA_DEVICE	1
 
+#define WL_DATA_DEVICE_MANAGER_CREATE_DATA_SOURCE_SINCE_VERSION	1
+#define WL_DATA_DEVICE_MANAGER_GET_DATA_DEVICE_SINCE_VERSION	1
+
 static inline void
 wl_data_device_manager_set_user_data(struct wl_data_device_manager *wl_data_device_manager, void *user_data)
 {
@@ -1013,6 +1103,12 @@ wl_data_device_manager_get_user_data(struct wl_data_device_manager *wl_data_devi
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_data_device_manager);
 }
 
+static inline uint32_t
+wl_data_device_manager_get_version(struct wl_data_device_manager *wl_data_device_manager)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_data_device_manager);
+}
+
 static inline void
 wl_data_device_manager_destroy(struct wl_data_device_manager *wl_data_device_manager)
 {
@@ -1043,6 +1139,8 @@ wl_data_device_manager_get_data_device(struct wl_data_device_manager *wl_data_de
 
 #define WL_SHELL_GET_SHELL_SURFACE	0
 
+#define WL_SHELL_GET_SHELL_SURFACE_SINCE_VERSION	1
+
 static inline void
 wl_shell_set_user_data(struct wl_shell *wl_shell, void *user_data)
 {
@@ -1055,6 +1153,12 @@ wl_shell_get_user_data(struct wl_shell *wl_shell)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_shell);
 }
 
+static inline uint32_t
+wl_shell_get_version(struct wl_shell *wl_shell)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_shell);
+}
+
 static inline void
 wl_shell_destroy(struct wl_shell *wl_shell)
 {
@@ -1232,6 +1336,17 @@ wl_shell_surface_add_listener(struct wl_shell_surface *wl_shell_surface,
 #define WL_SHELL_SURFACE_SET_TITLE	8
 #define WL_SHELL_SURFACE_SET_CLASS	9
 
+#define WL_SHELL_SURFACE_PONG_SINCE_VERSION	1
+#define WL_SHELL_SURFACE_MOVE_SINCE_VERSION	1
+#define WL_SHELL_SURFACE_RESIZE_SINCE_VERSION	1
+#define WL_SHELL_SURFACE_SET_TOPLEVEL_SINCE_VERSION	1
+#define WL_SHELL_SURFACE_SET_TRANSIENT_SINCE_VERSION	1
+#define WL_SHELL_SURFACE_SET_FULLSCREEN_SINCE_VERSION	1
+#define WL_SHELL_SURFACE_SET_POPUP_SINCE_VERSION	1
+#define WL_SHELL_SURFACE_SET_MAXIMIZED_SINCE_VERSION	1
+#define WL_SHELL_SURFACE_SET_TITLE_SINCE_VERSION	1
+#define WL_SHELL_SURFACE_SET_CLASS_SINCE_VERSION	1
+
 static inline void
 wl_shell_surface_set_user_data(struct wl_shell_surface *wl_shell_surface, void *user_data)
 {
@@ -1244,6 +1359,12 @@ wl_shell_surface_get_user_data(struct wl_shell_surface *wl_shell_surface)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_shell_surface);
 }
 
+static inline uint32_t
+wl_shell_surface_get_version(struct wl_shell_surface *wl_shell_surface)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_shell_surface);
+}
+
 static inline void
 wl_shell_surface_destroy(struct wl_shell_surface *wl_shell_surface)
 {
@@ -1382,6 +1503,16 @@ wl_surface_add_listener(struct wl_surface *wl_surface,
 #define WL_SURFACE_SET_BUFFER_TRANSFORM	7
 #define WL_SURFACE_SET_BUFFER_SCALE	8
 
+#define WL_SURFACE_DESTROY_SINCE_VERSION	1
+#define WL_SURFACE_ATTACH_SINCE_VERSION	1
+#define WL_SURFACE_DAMAGE_SINCE_VERSION	1
+#define WL_SURFACE_FRAME_SINCE_VERSION	1
+#define WL_SURFACE_SET_OPAQUE_REGION_SINCE_VERSION	1
+#define WL_SURFACE_SET_INPUT_REGION_SINCE_VERSION	1
+#define WL_SURFACE_COMMIT_SINCE_VERSION	1
+#define WL_SURFACE_SET_BUFFER_TRANSFORM_SINCE_VERSION	2
+#define WL_SURFACE_SET_BUFFER_SCALE_SINCE_VERSION	3
+
 static inline void
 wl_surface_set_user_data(struct wl_surface *wl_surface, void *user_data)
 {
@@ -1394,6 +1525,12 @@ wl_surface_get_user_data(struct wl_surface *wl_surface)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_surface);
 }
 
+static inline uint32_t
+wl_surface_get_version(struct wl_surface *wl_surface)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_surface);
+}
+
 static inline void
 wl_surface_destroy(struct wl_surface *wl_surface)
 {
@@ -1529,6 +1666,10 @@ wl_seat_add_listener(struct wl_seat *wl_seat,
 #define WL_SEAT_GET_KEYBOARD	1
 #define WL_SEAT_GET_TOUCH	2
 
+#define WL_SEAT_GET_POINTER_SINCE_VERSION	1
+#define WL_SEAT_GET_KEYBOARD_SINCE_VERSION	1
+#define WL_SEAT_GET_TOUCH_SINCE_VERSION	1
+
 static inline void
 wl_seat_set_user_data(struct wl_seat *wl_seat, void *user_data)
 {
@@ -1541,6 +1682,12 @@ wl_seat_get_user_data(struct wl_seat *wl_seat)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_seat);
 }
 
+static inline uint32_t
+wl_seat_get_version(struct wl_seat *wl_seat)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_seat);
+}
+
 static inline void
 wl_seat_destroy(struct wl_seat *wl_seat)
 {
@@ -1737,6 +1884,9 @@ wl_pointer_add_listener(struct wl_pointer *wl_pointer,
 #define WL_POINTER_SET_CURSOR	0
 #define WL_POINTER_RELEASE	1
 
+#define WL_POINTER_SET_CURSOR_SINCE_VERSION	1
+#define WL_POINTER_RELEASE_SINCE_VERSION	3
+
 static inline void
 wl_pointer_set_user_data(struct wl_pointer *wl_pointer, void *user_data)
 {
@@ -1749,6 +1899,12 @@ wl_pointer_get_user_data(struct wl_pointer *wl_pointer)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_pointer);
 }
 
+static inline uint32_t
+wl_pointer_get_version(struct wl_pointer *wl_pointer)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_pointer);
+}
+
 static inline void
 wl_pointer_destroy(struct wl_pointer *wl_pointer)
 {
@@ -1904,6 +2060,8 @@ wl_keyboard_add_listener(struct wl_keyboard *wl_keyboard,
 
 #define WL_KEYBOARD_RELEASE	0
 
+#define WL_KEYBOARD_RELEASE_SINCE_VERSION	3
+
 static inline void
 wl_keyboard_set_user_data(struct wl_keyboard *wl_keyboard, void *user_data)
 {
@@ -1916,6 +2074,12 @@ wl_keyboard_get_user_data(struct wl_keyboard *wl_keyboard)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_keyboard);
 }
 
+static inline uint32_t
+wl_keyboard_get_version(struct wl_keyboard *wl_keyboard)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_keyboard);
+}
+
 static inline void
 wl_keyboard_destroy(struct wl_keyboard *wl_keyboard)
 {
@@ -2032,6 +2196,8 @@ wl_touch_add_listener(struct wl_touch *wl_touch,
 
 #define WL_TOUCH_RELEASE	0
 
+#define WL_TOUCH_RELEASE_SINCE_VERSION	3
+
 static inline void
 wl_touch_set_user_data(struct wl_touch *wl_touch, void *user_data)
 {
@@ -2044,6 +2210,12 @@ wl_touch_get_user_data(struct wl_touch *wl_touch)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_touch);
 }
 
+static inline uint32_t
+wl_touch_get_version(struct wl_touch *wl_touch)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_touch);
+}
+
 static inline void
 wl_touch_destroy(struct wl_touch *wl_touch)
 {
@@ -2248,6 +2420,7 @@ wl_output_add_listener(struct wl_output *wl_output,
 				     (void (**)(void)) listener, data);
 }
 
+
 static inline void
 wl_output_set_user_data(struct wl_output *wl_output, void *user_data)
 {
@@ -2260,6 +2433,12 @@ wl_output_get_user_data(struct wl_output *wl_output)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_output);
 }
 
+static inline uint32_t
+wl_output_get_version(struct wl_output *wl_output)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_output);
+}
+
 static inline void
 wl_output_destroy(struct wl_output *wl_output)
 {
@@ -2270,6 +2449,10 @@ wl_output_destroy(struct wl_output *wl_output)
 #define WL_REGION_ADD	1
 #define WL_REGION_SUBTRACT	2
 
+#define WL_REGION_DESTROY_SINCE_VERSION	1
+#define WL_REGION_ADD_SINCE_VERSION	1
+#define WL_REGION_SUBTRACT_SINCE_VERSION	1
+
 static inline void
 wl_region_set_user_data(struct wl_region *wl_region, void *user_data)
 {
@@ -2282,6 +2465,12 @@ wl_region_get_user_data(struct wl_region *wl_region)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_region);
 }
 
+static inline uint32_t
+wl_region_get_version(struct wl_region *wl_region)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_region);
+}
+
 static inline void
 wl_region_destroy(struct wl_region *wl_region)
 {
@@ -2315,6 +2504,9 @@ enum wl_subcompositor_error {
 #define WL_SUBCOMPOSITOR_DESTROY	0
 #define WL_SUBCOMPOSITOR_GET_SUBSURFACE	1
 
+#define WL_SUBCOMPOSITOR_DESTROY_SINCE_VERSION	1
+#define WL_SUBCOMPOSITOR_GET_SUBSURFACE_SINCE_VERSION	1
+
 static inline void
 wl_subcompositor_set_user_data(struct wl_subcompositor *wl_subcompositor, void *user_data)
 {
@@ -2327,6 +2519,12 @@ wl_subcompositor_get_user_data(struct wl_subcompositor *wl_subcompositor)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_subcompositor);
 }
 
+static inline uint32_t
+wl_subcompositor_get_version(struct wl_subcompositor *wl_subcompositor)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_subcompositor);
+}
+
 static inline void
 wl_subcompositor_destroy(struct wl_subcompositor *wl_subcompositor)
 {
@@ -2361,6 +2559,13 @@ enum wl_subsurface_error {
 #define WL_SUBSURFACE_SET_SYNC	4
 #define WL_SUBSURFACE_SET_DESYNC	5
 
+#define WL_SUBSURFACE_DESTROY_SINCE_VERSION	1
+#define WL_SUBSURFACE_SET_POSITION_SINCE_VERSION	1
+#define WL_SUBSURFACE_PLACE_ABOVE_SINCE_VERSION	1
+#define WL_SUBSURFACE_PLACE_BELOW_SINCE_VERSION	1
+#define WL_SUBSURFACE_SET_SYNC_SINCE_VERSION	1
+#define WL_SUBSURFACE_SET_DESYNC_SINCE_VERSION	1
+
 static inline void
 wl_subsurface_set_user_data(struct wl_subsurface *wl_subsurface, void *user_data)
 {
@@ -2373,6 +2578,12 @@ wl_subsurface_get_user_data(struct wl_subsurface *wl_subsurface)
 	return wl_proxy_get_user_data((struct wl_proxy *) wl_subsurface);
 }
 
+static inline uint32_t
+wl_subsurface_get_version(struct wl_subsurface *wl_subsurface)
+{
+	return wl_proxy_get_version((struct wl_proxy *) wl_subsurface);
+}
+
 static inline void
 wl_subsurface_destroy(struct wl_subsurface *wl_subsurface)
 {
