@@ -1,9 +1,9 @@
-/* 
+/*
  * Copyright © 2008-2013 Kristian Høgsberg
  * Copyright © 2013      Rafael Antognolli
  * Copyright © 2013      Jasper St. Pierre
  * Copyright © 2010-2013 Intel Corporation
- * 
+ *
  * Permission to use, copy, modify, distribute, and sell this
  * software and its documentation for any purpose is hereby granted
  * without fee, provided that the above copyright notice appear in
@@ -15,7 +15,7 @@
  * representations about the suitability of this software for any
  * purpose.  It is provided "as is" without express or implied
  * warranty.
- * 
+ *
  * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
  * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
  * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
@@ -62,23 +62,33 @@ extern const struct wl_interface xdg_popup_interface;
  * static_assert to ensure the protocol and implementation versions match.
  */
 enum xdg_shell_version {
-	XDG_SHELL_VERSION_CURRENT = 4,
+	XDG_SHELL_VERSION_CURRENT = 5,
 };
 #endif /* XDG_SHELL_VERSION_ENUM */
 
+#ifndef XDG_SHELL_ERROR_ENUM
+#define XDG_SHELL_ERROR_ENUM
+enum xdg_shell_error {
+	XDG_SHELL_ERROR_ROLE = 0,
+	XDG_SHELL_ERROR_DEFUNCT_SURFACES = 1,
+	XDG_SHELL_ERROR_NOT_THE_TOPMOST_POPUP = 2,
+	XDG_SHELL_ERROR_INVALID_POPUP_PARENT = 3,
+};
+#endif /* XDG_SHELL_ERROR_ENUM */
+
 /**
  * xdg_shell - create desktop-style surfaces
  * @ping: check if the client is alive
  *
- * This interface is implemented by servers that provide desktop-style
- * user interfaces.
- *
- * It allows clients to associate a xdg_surface with a basic surface.
+ * xdg_shell allows clients to turn a wl_surface into a "real window"
+ * which can be dragged, resized, stacked, and moved around by the user.
+ * Everything about this interface is suited towards traditional desktop
+ * environments.
  */
 struct xdg_shell_listener {
 	/**
 	 * ping - check if the client is alive
-	 * @serial: pass this to the callback
+	 * @serial: pass this to the pong request
 	 *
 	 * The ping event asks the client if it's still alive. Pass the
 	 * serial specified in the event back to the compositor by sending
@@ -88,6 +98,9 @@ struct xdg_shell_listener {
 	 * alive. It's unspecified what will happen if the client doesn't
 	 * respond to the ping request, or in what timeframe. Clients
 	 * should try to respond in a reasonable amount of time.
+	 *
+	 * A compositor is free to ping in any way it wants, but a client
+	 * must always respond to any xdg_shell object it created.
 	 */
 	void (*ping)(void *data,
 		     struct xdg_shell *xdg_shell,
@@ -102,10 +115,17 @@ xdg_shell_add_listener(struct xdg_shell *xdg_shell,
 				     (void (**)(void)) listener, data);
 }
 
-#define XDG_SHELL_USE_UNSTABLE_VERSION	0
-#define XDG_SHELL_GET_XDG_SURFACE	1
-#define XDG_SHELL_GET_XDG_POPUP	2
-#define XDG_SHELL_PONG	3
+#define XDG_SHELL_DESTROY	0
+#define XDG_SHELL_USE_UNSTABLE_VERSION	1
+#define XDG_SHELL_GET_XDG_SURFACE	2
+#define XDG_SHELL_GET_XDG_POPUP	3
+#define XDG_SHELL_PONG	4
+
+#define XDG_SHELL_DESTROY_SINCE_VERSION	1
+#define XDG_SHELL_USE_UNSTABLE_VERSION_SINCE_VERSION	1
+#define XDG_SHELL_GET_XDG_SURFACE_SINCE_VERSION	1
+#define XDG_SHELL_GET_XDG_POPUP_SINCE_VERSION	1
+#define XDG_SHELL_PONG_SINCE_VERSION	1
 
 static inline void
 xdg_shell_set_user_data(struct xdg_shell *xdg_shell, void *user_data)
@@ -119,9 +139,18 @@ xdg_shell_get_user_data(struct xdg_shell *xdg_shell)
 	return wl_proxy_get_user_data((struct wl_proxy *) xdg_shell);
 }
 
+static inline uint32_t
+xdg_shell_get_version(struct xdg_shell *xdg_shell)
+{
+	return wl_proxy_get_version((struct wl_proxy *) xdg_shell);
+}
+
 static inline void
 xdg_shell_destroy(struct xdg_shell *xdg_shell)
 {
+	wl_proxy_marshal((struct wl_proxy *) xdg_shell,
+			 XDG_SHELL_DESTROY);
+
 	wl_proxy_destroy((struct wl_proxy *) xdg_shell);
 }
 
@@ -144,12 +173,12 @@ xdg_shell_get_xdg_surface(struct xdg_shell *xdg_shell, struct wl_surface *surfac
 }
 
 static inline struct xdg_popup *
-xdg_shell_get_xdg_popup(struct xdg_shell *xdg_shell, struct wl_surface *surface, struct wl_surface *parent, struct wl_seat *seat, uint32_t serial, int32_t x, int32_t y, uint32_t flags)
+xdg_shell_get_xdg_popup(struct xdg_shell *xdg_shell, struct wl_surface *surface, struct wl_surface *parent, struct wl_seat *seat, uint32_t serial, int32_t x, int32_t y)
 {
 	struct wl_proxy *id;
 
 	id = wl_proxy_marshal_constructor((struct wl_proxy *) xdg_shell,
-			 XDG_SHELL_GET_XDG_POPUP, &xdg_popup_interface, NULL, surface, parent, seat, serial, x, y, flags);
+			 XDG_SHELL_GET_XDG_POPUP, &xdg_popup_interface, NULL, surface, parent, seat, serial, x, y);
 
 	return (struct xdg_popup *) id;
 }
@@ -229,7 +258,7 @@ enum xdg_surface_state {
 #endif /* XDG_SURFACE_STATE_ENUM */
 
 /**
- * xdg_surface - desktop-style metadata interface
+ * xdg_surface - A desktop window
  * @configure: suggest a surface change
  * @close: surface wants to be closed
  *
@@ -240,9 +269,18 @@ enum xdg_surface_state {
  * properties like maximized, fullscreen, minimized, and to move and resize
  * them, and associate metadata like title and app id.
  *
- * On the server side the object is automatically destroyed when the
- * related wl_surface is destroyed. On client side, xdg_surface.destroy()
- * must be called before destroying the wl_surface object.
+ * The client must call wl_surface.commit on the corresponding wl_surface
+ * for the xdg_surface state to take effect. Prior to committing the new
+ * state, it can set up initial configuration, such as maximizing or
+ * setting a window geometry.
+ *
+ * Even without attaching a buffer the compositor must respond to initial
+ * committed configuration, for instance sending a configure event with
+ * expected window geometry if the client maximized its surface during
+ * initialization.
+ *
+ * For a surface to be mapped by the compositor the client must have
+ * committed both an xdg_surface state and a buffer.
  */
 struct xdg_surface_listener {
 	/**
@@ -252,16 +290,27 @@ struct xdg_surface_listener {
 	 * @states: (none)
 	 * @serial: (none)
 	 *
-	 * The configure event asks the client to resize its surface.
+	 * The configure event asks the client to resize its surface or
+	 * to change its state.
 	 *
 	 * The width and height arguments specify a hint to the window
 	 * about how its surface should be resized in window geometry
-	 * coordinates. The states listed in the event specify how the
-	 * width/height arguments should be interpreted.
+	 * coordinates. See set_window_geometry.
 	 *
-	 * A client should arrange a new surface, and then send a
-	 * ack_configure request with the serial sent in this configure
-	 * event before attaching a new surface.
+	 * If the width or height arguments are zero, it means the client
+	 * should decide its own window dimension. This may happen when the
+	 * compositor need to configure the state of the surface but
+	 * doesn't have any information about any previous or expected
+	 * dimension.
+	 *
+	 * The states listed in the event specify how the width/height
+	 * arguments should be interpreted, and possibly how it should be
+	 * drawn.
+	 *
+	 * Clients should arrange their surface for the new size and
+	 * states, and then send a ack_configure request with the serial
+	 * sent in this configure event at some point before committing the
+	 * new surface.
 	 *
 	 * If the client receives multiple configure events before it can
 	 * respond to one, it is free to discard all but the last event it
@@ -312,6 +361,21 @@ xdg_surface_add_listener(struct xdg_surface *xdg_surface,
 #define XDG_SURFACE_UNSET_FULLSCREEN	12
 #define XDG_SURFACE_SET_MINIMIZED	13
 
+#define XDG_SURFACE_DESTROY_SINCE_VERSION	1
+#define XDG_SURFACE_SET_PARENT_SINCE_VERSION	1
+#define XDG_SURFACE_SET_TITLE_SINCE_VERSION	1
+#define XDG_SURFACE_SET_APP_ID_SINCE_VERSION	1
+#define XDG_SURFACE_SHOW_WINDOW_MENU_SINCE_VERSION	1
+#define XDG_SURFACE_MOVE_SINCE_VERSION	1
+#define XDG_SURFACE_RESIZE_SINCE_VERSION	1
+#define XDG_SURFACE_ACK_CONFIGURE_SINCE_VERSION	1
+#define XDG_SURFACE_SET_WINDOW_GEOMETRY_SINCE_VERSION	1
+#define XDG_SURFACE_SET_MAXIMIZED_SINCE_VERSION	1
+#define XDG_SURFACE_UNSET_MAXIMIZED_SINCE_VERSION	1
+#define XDG_SURFACE_SET_FULLSCREEN_SINCE_VERSION	1
+#define XDG_SURFACE_UNSET_FULLSCREEN_SINCE_VERSION	1
+#define XDG_SURFACE_SET_MINIMIZED_SINCE_VERSION	1
+
 static inline void
 xdg_surface_set_user_data(struct xdg_surface *xdg_surface, void *user_data)
 {
@@ -324,6 +388,12 @@ xdg_surface_get_user_data(struct xdg_surface *xdg_surface)
 	return wl_proxy_get_user_data((struct wl_proxy *) xdg_surface);
 }
 
+static inline uint32_t
+xdg_surface_get_version(struct xdg_surface *xdg_surface)
+{
+	return wl_proxy_get_version((struct wl_proxy *) xdg_surface);
+}
+
 static inline void
 xdg_surface_destroy(struct xdg_surface *xdg_surface)
 {
@@ -334,7 +404,7 @@ xdg_surface_destroy(struct xdg_surface *xdg_surface)
 }
 
 static inline void
-xdg_surface_set_parent(struct xdg_surface *xdg_surface, struct wl_surface *parent)
+xdg_surface_set_parent(struct xdg_surface *xdg_surface, struct xdg_surface *parent)
 {
 	wl_proxy_marshal((struct wl_proxy *) xdg_surface,
 			 XDG_SURFACE_SET_PARENT, parent);
@@ -425,40 +495,65 @@ xdg_surface_set_minimized(struct xdg_surface *xdg_surface)
 }
 
 /**
- * xdg_popup - desktop-style metadata interface
+ * xdg_popup - short-lived, popup surfaces for menus
  * @popup_done: popup interaction is done
  *
- * An interface that may be implemented by a wl_surface, for
- * implementations that provide a desktop-style popups/menus. A popup
- * surface is a transient surface with an added pointer grab.
+ * A popup surface is a short-lived, temporary surface that can be used
+ * to implement menus. It takes an explicit grab on the surface that will
+ * be dismissed when the user dismisses the popup. This can be done by the
+ * user clicking outside the surface, using the keyboard, or even locking
+ * the screen through closing the lid or a timeout.
  *
- * An existing implicit grab will be changed to owner-events mode, and the
- * popup grab will continue after the implicit grab ends (i.e. releasing
- * the mouse button does not cause the popup to be unmapped).
+ * When the popup is dismissed, a popup_done event will be sent out, and at
+ * the same time the surface will be unmapped. The xdg_popup object is now
+ * inert and cannot be reactivated, so clients should destroy it.
+ * Explicitly destroying the xdg_popup object will also dismiss the popup
+ * and unmap the surface.
  *
- * The popup grab continues until the window is destroyed or a mouse button
- * is pressed in any other clients window. A click in any of the clients
- * surfaces is reported as normal, however, clicks in other clients
- * surfaces will be discarded and trigger the callback.
+ * Clients will receive events for all their surfaces during this grab
+ * (which is an "owner-events" grab in X11 parlance). This is done so that
+ * users can navigate through submenus and other "nested" popup windows
+ * without having to dismiss the topmost popup.
  *
- * The x and y arguments specify the locations of the upper left corner of
- * the surface relative to the upper left corner of the parent surface, in
- * surface local coordinates.
+ * Clients that want to dismiss the popup when another surface of their own
+ * is clicked should dismiss the popup using the destroy request.
  *
- * xdg_popup surfaces are always transient for another surface.
+ * The parent surface must have either an xdg_surface or xdg_popup role.
+ *
+ * Specifying an xdg_popup for the parent means that the popups are nested,
+ * with this popup now being the topmost popup. Nested popups must be
+ * destroyed in the reverse order they were created in, e.g. the only popup
+ * you are allowed to destroy at all times is the topmost one.
+ *
+ * If there is an existing popup when creating a new popup, the parent must
+ * be the current topmost popup.
+ *
+ * A parent surface must be mapped before the new popup is mapped.
+ *
+ * When compositors choose to dismiss a popup, they will likely dismiss
+ * every nested popup as well. When a compositor dismisses popups, it will
+ * follow the same dismissing order as required from the client.
+ *
+ * The x and y arguments passed when creating the popup object specify
+ * where the top left of the popup should be placed, relative to the local
+ * surface coordinates of the parent surface. See xdg_shell.get_xdg_popup.
+ *
+ * The client must call wl_surface.commit on the corresponding wl_surface
+ * for the xdg_popup state to take effect.
+ *
+ * For a surface to be mapped by the compositor the client must have
+ * committed both the xdg_popup state and a buffer.
  */
 struct xdg_popup_listener {
 	/**
 	 * popup_done - popup interaction is done
-	 * @serial: serial of the implicit grab on the pointer
 	 *
-	 * The popup_done event is sent out when a popup grab is broken,
-	 * that is, when the users clicks a surface that doesn't belong to
-	 * the client owning the popup surface.
+	 * The popup_done event is sent out when a popup is dismissed by
+	 * the compositor. The client should destroy the xdg_popup object
+	 * at this point.
 	 */
 	void (*popup_done)(void *data,
-			   struct xdg_popup *xdg_popup,
-			   uint32_t serial);
+			   struct xdg_popup *xdg_popup);
 };
 
 static inline int
@@ -471,6 +566,8 @@ xdg_popup_add_listener(struct xdg_popup *xdg_popup,
 
 #define XDG_POPUP_DESTROY	0
 
+#define XDG_POPUP_DESTROY_SINCE_VERSION	1
+
 static inline void
 xdg_popup_set_user_data(struct xdg_popup *xdg_popup, void *user_data)
 {
@@ -483,6 +580,12 @@ xdg_popup_get_user_data(struct xdg_popup *xdg_popup)
 	return wl_proxy_get_user_data((struct wl_proxy *) xdg_popup);
 }
 
+static inline uint32_t
+xdg_popup_get_version(struct xdg_popup *xdg_popup)
+{
+	return wl_proxy_get_version((struct wl_proxy *) xdg_popup);
+}
+
 static inline void
 xdg_popup_destroy(struct xdg_popup *xdg_popup)
 {
